#include "LICENSE.h"
#include "mex.h"
#include "eyta.h"

#define ANT_AREOR 1000
#define AREA_SIZE 16384
#define INTERP(i, f1, f2) (a = (f1), i + (a - u)/(a - (f2)))

extern char konf[128][4][6];

/* Custom memory allocation routine.  */
/* Makes it unnessary to free each allocated item (a single call with size equal
   -1 frees everything.) Good for mex file use to avoid memory leaks.
*/
void *allok(int size, char *first_free)
{
    static int akt_area;
    static char *area[ANT_AREOR];
    int i;

    if (size < 0) {
	for (i = 0; i < ANT_AREOR; i++)
	    if (area[i])
		free(area[i]);
	return NULL;
    }
    if (first_free == NULL) {
	for (i = 0; i < ANT_AREOR; i++)
	    area[i] = NULL;
        akt_area = 0;
	area[akt_area] = malloc(AREA_SIZE);
	if (!area[akt_area])
	    return NULL;
	first_free = area[akt_area];
    }
    if (AREA_SIZE - (first_free - area[akt_area]) < size) {
	akt_area++;
	if (akt_area >= ANT_AREOR || !(area[akt_area] = malloc(AREA_SIZE)))
	    return NULL;
	first_free = area[akt_area];
    }
    return first_free;
}

struct polygon **listcontour3d(double *d, double u, int in, int jn, int kn,
			      int i1, int i2, int j1, int j2, int k1, int k2,
			      struct polygon **poly)
{
    int i, j, k;
    int left = -(j2 - j1 + 2);
    int down = -(j2 - j1 + 2)*(i2 - i1 + 2);
    int up = jn*in;
    unsigned char *maparea, *map;
    double *kol, a;
/*    struct polygon *head = NULL, **poly = &head;	*/
    struct point *horn = NULL;

    maparea = calloc(-down * (k2 - k1 + 2), sizeof(char));
    if (!maparea)
	return NULL;
    map = maparea-down-left+1;	/* The beginning is not used */
    for (k = k1; k <= k2; k++, map -= left)
	for (i = i1; i <= i2; i++, map++) {
	    kol = d + k*in*jn + i*jn;
	    for (j = j1; j <= j2; j++, map++)
		if (kol[j] >= u) {
		    map[0] |= 128;
		    map[-1] |= 32;
		    map[left] |= 8;
		    map[left-1] |= 2;
		    map[down] |= 64;
		    map[down-1] |= 16;
		    map[down+left] |= 4;
		    map[down+left-1] |= 1;
		}
	}
    map = maparea-down-left+1;
    for (k = k1; k < k2; k++, map -= 2*left)
        for (i = i1; i < i2; i++, map += 2) {
            kol = d + k*in*jn + i*jn;
            for (j = j1; j < j2; j++, map++) {
		int p, sida;
		unsigned char index = *map;
		if (index >= 128)
		    index ^= 255;
		if (!index)
		    continue;
		for (p = 0; p < 4 && konf[index][p][1]; p++) {
		    *poly = allok(sizeof(struct polygon), (char *)horn);
		    if (!*poly)
			return NULL;
		    horn = (*poly)->horn;
		    for (sida = 0; sida < 6 && konf[index][p][sida]; sida++) {
			switch (konf[index][p][sida]) {
			    case 1:
				horn -> x = INTERP(i, kol[j], kol[j+jn]);
				horn -> y = j;
				horn -> z = k;
				break;
			    case 2:
				horn -> x = INTERP(i, kol[j+up], kol[j+up+jn]);
				horn -> y = j;
				horn -> z = k + 1;
				break;
			    case 3:
			        horn -> x = INTERP(i, kol[j+1], kol[j+1+jn]);
				horn -> y = j + 1;
				horn -> z = k;
				break;
			    case 4:
			        horn -> x = INTERP(i, kol[j+1+up],
				    kol[j+1+jn+up]);
				horn -> y = j + 1;
				horn -> z = k + 1;
				break;
			    case 5:
			    	horn -> x = i;
				horn -> y = INTERP(j, kol[j], kol[j+1]);
				horn -> z = k;
				break;
			    case 6:
			    	horn -> x = i;
				horn -> y = INTERP(j, kol[j+up], kol[j+1+up]);
				horn -> z = k + 1;
				break;
			    case 7:
			    	horn -> x = i + 1;
				horn -> y = INTERP(j, kol[j+jn], kol[j+1+jn]);
				horn -> z = k;
				break;
			    case 8:
			        horn -> x = i + 1;
				horn -> y = INTERP(j, kol[j+jn+up],
				    kol[j+1+jn+up]);
				horn -> z = k + 1;
				break;
			    case 9:
			        horn -> x = i;
				horn -> y = j;
				horn -> z = INTERP(k, kol[j], kol[j+up]);
				break;
			    case 10:
			        horn -> x = i;
				horn -> y = j + 1;
				horn -> z = INTERP(k, kol[j+1], kol[j+1+up]);
				break;
			    case 11:
			        horn -> x = i + 1;
				horn -> y = j;
				horn -> z = INTERP(k, kol[j+jn], kol[j+jn+up]);
				break;
			    case 12:
			        horn -> x = i + 1;
				horn -> y = j + 1;
				horn -> z = INTERP(k, kol[j+1+jn],
				    kol[j+1+jn+up]);
				break;
			}
			horn++;
		    }
		    (*poly)->sidor = sida;
		    poly = &(*poly)->next;
		}
	    }
	}
    *poly = NULL;
    free(maparea);
    return poly;	/* Return the address of the last element's 'next' */
}			/* pointer */

char konf[128][4][6] = {
    {{0}},			/* 0 */
    {{4,8,12}},
    {{12,3,7}},
    {{4,8,7,3}},
    {{8,11,2}},
    {{4,12,11,2}},
    {{8,11,2}, {12,3,7}},
    {{4,2,11,7,3}},
    {{11,1,7}},
    {{4,8,12}, {1,7,11}},	/* korrigerad */
    {{11,1,3,12}},
    {{4,8,11,1,3}},
    {{8,2,1,7}},
    {{4,12,7,1,2}},
    {{8,2,1,3,12}},
    {{4,2,1,3}},
    {{4,6,10}},			/* 16 */
    {{6,10,12,8}},
    {{4,6,10}, {12,3,7}},
    {{6,10,3,7,8}},
    {{4,6,10}, {8,11,2}},
    {{6,10,12,11,2}},
    {{4,6,10}, {8,11,2}, {12,3,7}},
    {{6,10,3,7,11,2}},
    {{4,6,10}, {11,1,7}},
    {{1,7,11}, {6,8,12,10}},	/* korrigerad */
    {{4,6,10}, {11,1,3,12}},
    {{6,10,3,1,11,8}},
    {{4,6,10}, {8,2,1,7}},
    {{6,10,12,7,1,2}},
    {{4,6,10}, {8,2,1,3,12}},
    {{6,10,3,1,2}},
    {{10,3,5}},			/* 32 */
    {{4,8,12}, {10,5,3}},	/* korrigerad */
    {{10,5,7,12}},
    {{4,10,5,7,8}},
    {{8,11,2}, {10,3,5}},
    {{10,5,3}, {4,2,11,12}},	/* korrigerad */
    {{8,11,2}, {10,5,7,12}},
    {{4,10,5,7,11,2}},
    {{10,5,3}, {1,7,11}},	/* korrigerad */
    {{10,5,3}, {4,8,12}, {1,7,11}},  /* korrigerad */
    {{10,5,1,11,12}},
    {{4,10,5,1,11,8}},
    {{10,5,3}, {2,1,7,8}},	/* korrigerad */
    {{3,7,12}, {4,2,1,5,10}},	/* korrigerad */
    {{8,2,1,5,10,12}},
    {{4,10,5,1,2}},
    {{4,6,5,3}},		/* 48 */
    {{6,5,3,12,8}},
    {{4,6,5,7,12}},
    {{6,5,7,8}},
    {{2,11,8}, {6,5,3,4}},	/* korrigerad */
    {{6,2,11,12,3,5}},
    {{4,8,12}, {6,2,11,7,5}},	/* korrigerad */
    {{6,2,11,7,5}},
    {{1,7,11}, {6,5,3,4}},	/* korrigerad */
    {{1,7,11}, {6,5,3,12,8}},	/* korrigerad */
    {{4,6,5,1,11,12}},
    {{6,5,1,11,8}},
    {{2,1,3,4}, {6,5,7,8}},	/* korrigerad, skrande ytor */
    {{6,2,1,5}, {3,7,12}},      /* korrigerad */
    {{4,8,12}, {6,5,1,2}},      /* korrigerad */
    {{6,2,1,5}},
    {{6,9,2}},			/* 64 */
    {{6,9,2}, {4,8,12}},        /* korrigerad */
    {{6,9,2}, {12,3,7}},
    {{6,9,2}, {4,8,7,3}},	/* korrigerad */
    {{6,9,11,8}},
    {{4,6,9,11,12}},
    {{3,7,12}, {6,9,11,8}},	/* korrigerad */
    {{4,6,9,11,7,3}},
    {{6,9,2}, {1,7,11}},	/* korrigerad */
    {{6,9,2}, {4,8,12}, {1,7,11}}, /* korrigerad */
    {{6,9,2}, {1,11,12,3}},	/* korrigerad */
    {{6,9,2}, {4,8,11,1,3}},	/* korrigerad */
    {{6,9,1,7,8}},
    {{4,6,9,1,7,12}},
    {{6,9,1,3,12,8}},
    {{4,6,9,1,3}},
    {{4,10,9,2}},		/* 80 */
    {{8,2,9,10,12}},
    {{3,7,12}, {9,2,4,10}},	/* korrigerad */
    {{8,2,9,10,3,7}},
    {{4,10,9,11,8}},
    {{9,11,12,10}},
    {{4,8,12}, {10,9,11,7,3}},	/* korrigerad */
    {{9,11,7,3,10}},
    {{1,7,11}, {9,2,4,10}},	/* korrigerad */
    {{8,11,2}, {9,1,7,12,10}},
    {{2,1,3,4}, {11,12,10,9}},	/* korrigerad, skrande ytor */
    {{8,11,2}, {9,1,3,10}},
    {{4,10,9,1,7,8}},
    {{9,1,7,12,10}},
    {{4,8,12}, {9,10,3,1}},
    {{9,1,3,10}},
    {{6,9,2}, {5,3,10}},	/* 96, korrigerad */
    {{6,9,2}, {10,5,3}, {4,8,12}},  /* korrigerad */
    {{6,9,2}, {2,5,7,12}},	/* korrigerad */
    {{4,6,10}, {8,2,9,5,7}},
    {{10,5,3}, {6,9,11,8}},	/* korrigerad */
    {{6,10,4}, {5,9,11,12,3}},	/* korrigerad */
    {{8,6,5,7}, {11,12,10,9}},	/* korrigerad, skrande ytor */
    {{4,6,10}, {9,11,7,5}},     /* korrigerad */
    {{6,10,3,7,11,2}},
    {{4,6,10}, {8,11,2}, {9,1,5}, {12,3,7}},
    {{6,9,2}, {5,1,11,12,10}},	/* korrigerad */
    {{4,6,10}, {8,11,2}, {9,1,5}},
    {{10,5,3}, {6,9,1,7,8}},	/* korrigerad */
    {{4,6,10}, {9,1,5}, {12,3,7}},
    {{1,5,9}, {6,8,12,10}},	/* korrigerad */
    {{4,6,10}, {9,1,5}},
    {{4,2,9,5,3}},		/* 112 */
    {{8,2,9,5,3,12}},
    {{4,12,7,5,9,2}},
    {{8,2,9,5,7}},
    {{4,8,11,9,5,3}},
    {{9,5,3,12,11}},
    {{4,8,12}, {5,9,11,7}},	/* korrigerad */
    {{9,5,7,11}},
    {{1,7,11}, {4,2,9,5,3}},	/* korrigerad */
    {{8,11,2}, {9,1,5}, {12,3,7}},
    {{1,5,9}, {2,11,12,4}},	/* korrigerad */
    {{8,11,2}, {9,1,5}},
    {{1,5,9}, {3,7,8,4}},	/* korrigerad */
    {{9,1,5}, {12,3,7}},
    {{4,8,12}, {9,1,5}},
    {{9,1,5}}
};

void sort_poly(struct polygon *polylista, mxArray *plhs[])
{
    int antal[4] = {0,0,0,0};
    struct polygon *ptr = polylista;
    double *ptr_x[4], *ptr_y[4], *ptr_z[4], *tabs[4];
    int i, index;

    while (ptr) {
	antal[ptr -> sidor - 3]++;
	ptr = ptr->next;
    }
    for (i = 0; i < 4; i++) {
	plhs[i] = mxCreateDoubleMatrix(i + 3, antal[i] * 3, mxREAL);
	tabs[i] = mxGetPr(plhs[i]);
    }
    if ((antal[0] && !tabs[0]) || (antal[1] && !tabs[1]) || 
	(antal[2] && !tabs[2]) || (antal[3] && !tabs[3]))
	mexErrMsgTxt("Out of memory when allocating output matrices");

    for (i = 0; i < 4; i++) {
	ptr_x[i] = tabs[i];
	ptr_y[i] = tabs[i] + (i + 3) * antal[i];
	ptr_z[i] = tabs[i] + (i + 3) * 2 *antal[i];
    }
    ptr = polylista;
    while (ptr) {
	index = ptr -> sidor - 3;
	for (i = 0; i < index + 3; i++) {
	    *ptr_x[index]++ = ptr -> horn[i].x;
	    *ptr_y[index]++ = ptr -> horn[i].y;
	    *ptr_z[index]++ = ptr -> horn[i].z;
	}
	ptr = ptr->next;
    }
}

#define Mat prhs[0]
#define Lim prhs[1]
#define Const prhs[2]
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    int i1, i2, in, j1, j2, jn, k1, k2, kn, i, ant_konst;
    double *gr;
    int ndim;
    const int *dim_array;
    struct polygon *polylista = NULL, **next_ptr = &polylista;

    if (nrhs != 3 || nlhs != 4)
	mexErrMsgTxt("Usage: [p3,p4,p5,p6] = polygons(3Dmatrix, "
	    "limits, constant)");
    ndim = mxGetNumberOfDimensions(Mat);
    if (ndim != 3)
	mexErrMsgTxt("The first argument should be a 3D matrix");
    dim_array = mxGetDimensions(Mat);
/* meshgrid swaps the order of the two first indices by convention */
    in = dim_array[1];
    jn = dim_array[0];
    kn = dim_array[2];

    if (mxGetM(Lim) == 0) {
	i1 = 0;
	i2 = in - 1;
	j1 = 0;
	j2 = jn - 1;
	k1 = 0;
	k2 = kn - 1;
    } else if (mxGetM(Lim) * mxGetN(Lim) == 6) {
	gr = mxGetPr(Lim);
	i1 = gr[0] - 1;
	i2 = gr[1] - 1;
	j1 = gr[2] - 1;
	j2 = gr[3] - 1;
	k1 = gr[4] - 1;
	k2 = gr[5] - 1;
    } else
	mexErrMsgTxt("The limits vector should have six elements or be empty");

    if (mxGetM(Const) == 0)
	mexErrMsgTxt("No constant supplied");
    if (i1 < 0 || i1 >= i2 || i2 >= in ||
	j1 < 0 || j1 >= j2 || j2 >= jn ||
	k1 < 0 || k1 >= k2 || k2 >= kn)
	mexErrMsgTxt("The limits are wrong");

    ant_konst = mxGetM(Const) * mxGetN(Const);

    for (i = 0; i < ant_konst; i++)
	next_ptr = listcontour3d(mxGetPr(Mat), mxGetPr(Const)[i],
	    in, jn, kn, i1, i2, j1, j2, k1, k2, next_ptr);

    if (!polylista)
	mexErrMsgTxt("No output");
    sort_poly(polylista, plhs);
    allok(-1, NULL);	
}




