function [vel]=interpolate(v,range,elev,xr,zr,vf)

%interpolate(v,range,elev,xr,zr,vf): Interpolates polar data 
% onto a rectangular grid given by vectors xr and zr.
%Use: vr=interpolate(v,range,elev,xr,zr,vf)
% where v is the matrix to be interpolated
% range is a monotonically increasing vector of ranges (not
%    necessarily equal spacing)
% elev is like range, in degrees
% xr, zr are monotonically increasing vectors, equally spaced (though 
%    dx does not have to equal dz)
% vf is the folding velocity. If omitted then folding is not taken
% into account
% Values not interpolatable are set to NaN

if nargin==5, vf=0;end

vel=NaN;
[mm nn]=size(v);

%sort out grids etc

angles=elev(1:nn)';
X=range'*cos(angles'*pi/180);
Z=range'*sin(angles'*pi/180);

nx=length(xr);
nz=length(zr);

%make sure vel is of correct size
vel(nx,nz)=0;
vel=vel+NaN;

%which way was the scan done??
if angles(nn)>angles(1);
  scandir=+1;
else
  scandir=-1;
end

%start interpolating
clc;
for j=1:nz,
home;
rowsleft=nz-j
  for i=1:nx,
    x=xr(i);
    z=zr(j);
    r=(x^2+z^2)^0.5;
    el=atan2(z,x)*180/pi;
    
    if el<=max(elev)

      m=find(range<=r);
      m1=max(m);

      n=find(angles<=el);
      n1=(scandir==1)*max(n)+(scandir==-1)*min(n);

      m2=m1+1;
      m3=m1+1;
      m4=m1;
 
      n2=n1;
      n3=n1+scandir;
      n4=n1+scandir;

      if (m1>=1)&(m2<=mm)&(min([n1 n3])>=1)&(max([n1 n3])<=nn) 

        d1=((x-X(m1,n1))^2+(z-Z(m1,n1))^2)^(-0.5);
        d2=((x-X(m2,n2))^2+(z-Z(m2,n2))^2)^(-0.5);
        d3=((x-X(m3,n3))^2+(z-Z(m3,n3))^2)^(-0.5);
        d4=((x-X(m4,n4))^2+(z-Z(m4,n4))^2)^(-0.5);
        vel1=v(m1,n1);
        vel2=v(m2,n2);
        vel3=v(m3,n3);
        vel4=v(m4,n4);
        if vf~=0
          vel2=vel2+vf*((vel1-vel2)>vf/2)-vf*((vel2-vel1)>vf/2);
          vel3=vel3+vf*((vel1-vel3)>vf/2)-vf*((vel3-vel1)>vf/2);
          vel4=vel4+vf*((vel1-vel4)>vf/2)-vf*((vel4-vel1)>vf/2);
        end
        vel(i,j)=(d1*vel1+d2*vel2+d3*vel3+d4*vel4)/(d1+d2+d3+d4);
      end   %end of if
    end     %end of other if
  end       %end of interpolation x-loop
end         %end of interpolation y-loop







