/* lschil.c
 * Unix command-line tool for displaying the rasters that were taken on a 
 * particular date in chilbolton data files listed on the command line
 */
#include <stdio.h>
#include <glob.h>
#include <string.h>
#include "chil.h"

int main(int argc,char **argv) {
   FileList filelist;
   int datelen,i,pos=0,status;
   unsigned short int day=0,month=0,year=0;
   unsigned short int pday=0,pmonth=0,pyear=0;
   char isdate=0;
   glob_t globbuf;
   unsigned int GLOB_FLAG=0;
   FILE *sumfile;
   char tmp[255];

//   CHILSecondarySumDir("/export/anvil/anvil-01/swrhgnrj/sum");
   CHILSecondarySumDir("/export/peevee/peevee-03/swrhgnrj/sum");

   if (argc<2) {
      fprintf(stderr,"Usage: %s file_pattern1 [file_pattern2 ...]\n       %s -date yyyymmdd file1 [file2...]\n",argv[0],argv[0]);
      exit(0);
   }

   for (i=1;i<argc;i++) {
      if (strcmp(argv[i],"-date")==0) {
         isdate=1;
         break;
      }
      if ((status=glob(argv[i],GLOB_FLAG,NULL,&globbuf)) == GLOB_NOSPACE) {
         globfree(&globbuf);
         fprintf(stderr,"Out of memory\n");
         exit(-1);
      }
      GLOB_FLAG=GLOB_APPEND;
   }

   if (isdate) {
      if (i==1) {
         globfree(&globbuf);
         fprintf(stderr,"No files specified\n");
         exit(-1);
      }
      if (i==argc-1) {
         globfree(&globbuf);
         fprintf(stderr,"No date specified\n");
         exit(-1);
      }
      datelen=strlen(argv[++i]);
      if (datelen==6) {
         sscanf(argv[i],"%02hu%02hu%02hu",&year,&month,&day);
      }
      else if (datelen==8) {
         sscanf(argv[i],"%04hu%02hu%02hu",&year,&month,&day);
         year=year-1900;
      }
      else {
         globfree(&globbuf);
         fprintf(stderr,"Can't understand date");
         exit(-1);
      }
      CHILGetFileList(day,month,year,globbuf.gl_pathc,globbuf.gl_pathv,&filelist);
      for (i=0;i<filelist.n;i++) {
         fprintf(stdout,"%s %hu-%hu ",filelist.name[i],filelist.first[i],filelist.last[i]);
      }
      fprintf(stdout,"\n");
   }
   else {
      for (i=0;i<globbuf.gl_pathc;i++) {
         if ((sumfile=CHILOpenSummary(globbuf.gl_pathv[i])) != NULL) {
            fgets(tmp,255,sumfile); /* skip first line */
            while(fgets(tmp, 255, sumfile) != '\0') {
               sscanf(tmp, "%*5d %*3hu %*8c %2hu/%2hu/%2hu",&day,&month,&year);
               if (day != pday || month != pmonth || year != pyear) {
                  fprintf(stdout,"%02hu%02hu%02hu \n",year,month,day);
                  pday=day;pmonth=month;pyear=year;
               }
            }
            fclose(sumfile);
         }
      }
   }
   globfree(&globbuf);
}
